@@ -49,7 +49,7 @@
 
 int default_read_method(int, char *, int);
 int default_write_method(int, const char *, int);
-#ifdef _SQUID_MSWIN_
+#if _SQUID_MSWIN_
 int socket_read_method(int, char *, int);
 int socket_write_method(int, const char *, int);
 int file_read_method(int, char *, int);
@@ -105,6 +105,9 @@ fd_close(int fd)
 {
     fde *F = &fd_table[fd];
 
+    assert(fd >= 0);
+    assert(F->flags.open == 1);
+
     if (F->type == FD_FILE) {
         assert(F->read_handler == NULL);
         assert(F->write_handler == NULL);
@@ -119,7 +122,7 @@ fd_close(int fd)
     *F = fde();
 }
 
-#ifdef _SQUID_MSWIN_
+#if _SQUID_MSWIN_
 
 int
 socket_read_method(int fd, char *buf, int len)
@@ -219,7 +222,7 @@ fd_open(int fd, unsigned int type, const char *desc)
     F->type = type;
     F->flags.open = 1;
     F->epoll_state = 0;
-#ifdef _SQUID_MSWIN_
+#if _SQUID_MSWIN_
 
     F->win32.handle = _get_osfhandle(fd);
 
