@@ -27,8 +27,8 @@ Ssl::Helper::~Helper()
 void Ssl::Helper::Init()
 {
     if (ssl_crtd == NULL)
-        ssl_crtd = helperCreate("ssl_crtd");
-    ssl_crtd->n_to_start = Ssl::TheConfig.ssl_crtd_n_running;
+        ssl_crtd = new helper("ssl_crtd");
+    ssl_crtd->childs = Ssl::TheConfig.ssl_crtdChildren;
     ssl_crtd->ipc_type = IPC_STREAM;
     // The crtd messages may contain the eol ('\n') character. We are
     // going to use the '\1' char as the end-of-message mark.
@@ -71,7 +71,7 @@ void Ssl::Helper::Shutdown()
     wordlistDestroy(&ssl_crtd->cmdline);
     if (!shutting_down)
         return;
-    helperFree(ssl_crtd);
+    delete ssl_crtd;
     ssl_crtd = NULL;
 }
 
@@ -79,7 +79,7 @@ void Ssl::Helper::sslSubmit(CrtdMessage const & message, HLPCB * callback, void
 {
     static time_t first_warn = 0;
 
-    if (ssl_crtd->stats.queue_size >= (int)(ssl_crtd->n_running * 2)) {
+    if (ssl_crtd->stats.queue_size >= (int)(ssl_crtd->childs.n_running * 2)) {
         if (first_warn == 0)
             first_warn = squid_curtime;
         if (squid_curtime - first_warn > 3 * 60)
