@@ -50,7 +50,7 @@ class WhoisState
 
 public:
     ~WhoisState();
-    void readReply (int fd, char *aBuffer, size_t aBufferLength, comm_err_t flag, int xerrno);
+    void readReply(const Comm::ConnectionPointer &, char *aBuffer, size_t aBufferLength, comm_err_t flag, int xerrno);
     void setReplyToOK(StoreEntry *sentry);
     StoreEntry *entry;
     HttpRequest *request;
@@ -60,7 +60,7 @@ public:
 };
 
 static PF whoisClose;
-static PF whoisTimeout;
+static CTCB whoisTimeout;
 static IOCB whoisReadReply;
 
 /* PUBLIC */
@@ -73,7 +73,7 @@ WhoisState::~WhoisState()
 }
 
 static void
-whoisWriteComplete(int fd, char *buf, size_t size, comm_err_t flag, int xerrno, void *data)
+whoisWriteComplete(const Comm::ConnectionPointer &, char *buf, size_t size, comm_err_t flag, int xerrno, void *data)
 {
     xfree(buf);
 }
@@ -82,7 +82,6 @@ void
 whoisStart(FwdState * fwd)
 {
     WhoisState *p;
-    int fd = fwd->server_fd;
     char *buf;
     size_t l;
     CBDATA_INIT_TYPE(WhoisState);
@@ -93,7 +92,7 @@ whoisStart(FwdState * fwd)
     p->dataWritten = false;
 
     p->entry->lock();
-    comm_add_close_handler(fd, whoisClose, p);
+    comm_add_close_handler(fwd->serverConnection()->fd, whoisClose, p);
 
     l = p->request->urlpath.size() + 3;
 
@@ -102,29 +101,32 @@ whoisStart(FwdState * fwd)
     String str_print=p->request->urlpath.substr(1,p->request->urlpath.size());
     snprintf(buf, l, SQUIDSTRINGPH"\r\n", SQUIDSTRINGPRINT(str_print));
 
-    AsyncCall::Pointer call = commCbCall(5,5, "whoisWriteComplete",
-                                         CommIoCbPtrFun(whoisWriteComplete, p));
-
-    Comm::Write(fd, buf, strlen(buf), call, NULL);
-    comm_read(fd, p->buf, BUFSIZ, whoisReadReply, p);
-    commSetTimeout(fd, Config.Timeout.read, whoisTimeout, p);
+    AsyncCall::Pointer writeCall = commCbCall(5,5, "whoisWriteComplete",
+                                   CommIoCbPtrFun(whoisWriteComplete, p));
+    Comm::Write(fwd->serverConnection(), buf, strlen(buf), writeCall, NULL);
+    AsyncCall::Pointer readCall = commCbCall(5,4, "whoisReadReply",
+                                  CommIoCbPtrFun(whoisReadReply, p));
+    comm_read(fwd->serverConnection(), p->buf, BUFSIZ, readCall);
+    AsyncCall::Pointer timeoutCall = commCbCall(5, 4, "whoisTimeout",
+                                     CommTimeoutCbPtrFun(whoisTimeout, p));
+    commSetConnTimeout(fwd->serverConnection(), Config.Timeout.read, timeoutCall);
 }
 
 /* PRIVATE */
 
 static void
-whoisTimeout(int fd, void *data)
+whoisTimeout(const CommTimeoutCbParams &io)
 {
-    WhoisState *p = (WhoisState *)data;
-    debugs(75, 1, "whoisTimeout: " << p->entry->url()  );
-    whoisClose(fd, p);
+    WhoisState *p = static_cast<WhoisState *>(io.data);
+    debugs(75, 3, HERE << io.conn << ", URL " << p->entry->url());
+    io.conn->close();
 }
 
 static void
-whoisReadReply(int fd, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
+whoisReadReply(const Comm::ConnectionPointer &conn, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
 {
     WhoisState *p = (WhoisState *)data;
-    p->readReply(fd, buf, len, flag, xerrno);
+    p->readReply(conn, buf, len, flag, xerrno);
 }
 
 void
@@ -137,27 +139,29 @@ WhoisState::setReplyToOK(StoreEntry *sentry)
 }
 
 void
-WhoisState::readReply (int fd, char *aBuffer, size_t aBufferLength, comm_err_t flag, int xerrno)
+WhoisState::readReply(const Comm::ConnectionPointer &conn, char *aBuffer, size_t aBufferLength, comm_err_t flag, int xerrno)
 {
     /* Bail out early on COMM_ERR_CLOSING - close handlers will tidy up for us */
     if (flag == COMM_ERR_CLOSING)
         return;
 
     aBuffer[aBufferLength] = '\0';
-    debugs(75, 3, "whoisReadReply: FD " << fd << " read " << aBufferLength << " bytes");
+    debugs(75, 3, HERE << conn << " read " << aBufferLength << " bytes");
     debugs(75, 5, "{" << aBuffer << "}");
 
     if (flag != COMM_OK) {
-        debugs(50, 2, "whoisReadReply: FD " << fd << ": read failure: " << xstrerror() << ".");
+        debugs(50, 2, HERE  << conn << ": read failure: " << xstrerror() << ".");
 
         if (ignoreErrno(errno)) {
-            comm_read(fd, aBuffer, BUFSIZ, whoisReadReply, this);
+            AsyncCall::Pointer call = commCbCall(5,4, "whoisReadReply",
+                                                 CommIoCbPtrFun(whoisReadReply, this));
+            comm_read(conn, aBuffer, BUFSIZ, call);
         } else {
             ErrorState *err;
             err = errorCon(ERR_READ_ERROR, HTTP_INTERNAL_SERVER_ERROR, fwd->request);
             err->xerrno = errno;
             fwd->fail(err);
-            comm_close(fd);
+            conn->close();
         }
         return;
     }
@@ -174,7 +178,9 @@ WhoisState::readReply (int fd, char *aBuffer, size_t aBufferLength, comm_err_t f
         entry->append(aBuffer, aBufferLength);
         entry->flush();
 
-        comm_read(fd, aBuffer, BUFSIZ, whoisReadReply, this);
+        AsyncCall::Pointer call = commCbCall(5,4, "whoisReadReply",
+                                             CommIoCbPtrFun(whoisReadReply, this));
+        comm_read(conn, aBuffer, BUFSIZ, call);
         return;
     }
 
@@ -187,7 +193,7 @@ WhoisState::readReply (int fd, char *aBuffer, size_t aBufferLength, comm_err_t f
 
     fwd->complete();
     debugs(75, 3, "whoisReadReply: Done: " << entry->url());
-    comm_close(fd);
+    conn->close();
 }
 
 static void
